//
//
//    2020  Dave Freese, W1HKJ
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <FL/filename.H>

#include "config.h"

#include "flio.h"
#include "flio_ui.h"
#include "flio_status.h"
#include "util.h"

Fl_Double_Window *mainwin = (Fl_Double_Window *)0;
Fl_Double_Window *commswin = (Fl_Double_Window *)0;
Fl_Double_Window *helpwin = (Fl_Double_Window *)0;

std::string HomeDir = "./";

status progstatus;

extern char input_buffer[2048];
extern char command[1024];

//----------------------------------------------------------------------
static void check_homedir(void)
{
	HomeDir.clear();
	char dirbuf[FL_PATH_MAX + 1];
	int r;
#ifdef __WIN32__
	fl_filename_expand(dirbuf, sizeof(dirbuf) -1, "$USERPROFILE/");
	HomeDir = dirbuf;
	HomeDir.append("flio.files\\");
	if ((r = mkdir(HomeDir.c_str()) == -1) && errno != EEXIST)
		exit(EXIT_FAILURE);
#else
	fl_filename_expand(dirbuf, FL_PATH_MAX, "$HOME/");
	HomeDir = dirbuf;
	HomeDir.append("flio.files/");
	if ((r = mkdir(HomeDir.c_str(), 0777)) == -1 && errno != EEXIST) {
		std:: cerr << "Could not make directory" << ' ' << HomeDir
				 << ": " << strerror(errno) << '\n';
		exit(EXIT_FAILURE);
	}
#endif
}

//----------------------------------------------------------------------

void cb_exit( void )
{
	progstatus.saveLastState();

	stop_digi_loop();
	close_xmlrpc();

	if (commswin) {
		commswin->hide();
		delete commswin;
	}

	mainwin->hide();
}

//----------------------------------------------------------------------
void cb_io_configure(void)
{
	commswin->show();
}

void init_comm_port()
{
	Init_Serial();
}

void close_comms_dialog()
{
	commswin->hide();
}

void cb_comms_dialog(void)
{
	commswin->hide();
}

void cb_about( void )
{
	helpwin->show();
}

void cb_helpwin(void)
{
	helpwin->hide();
}

void flio_init_widgets( void)
{
	init_comm_port();
}

std::string serial_buffer;
std::string send_buffer;
void update_flio_text(std::string s)
{
	flio_text->addstr (s);
	flio_text->redraw ();

	serial_buffer.append(s);

	if (send_buffer.empty()) {
		send_buffer = serial_buffer;
		serial_buffer.clear();
		Send_Serial (send_buffer);
		send_buffer.clear();
	}
}

int parse_args(int argc, char **argv, int& idx)
{
#ifdef __WIN32__
	return 1;
#endif

	std::string helpstr =
"Usage: \n\
  --help this help text\n\
  --version\n";

	if (strcasecmp("--help", argv[idx]) == 0) {
		std::cout << helpstr << std::endl;
		exit(0);
	} 
	if (strcasecmp("--version", argv[idx]) == 0) {
		std::string ver = "Version: ";
		ver.append(VERSION).append("\n");
		std::cout << ver;
		exit (0);
	}
	return 1;
}

int main(int argc, char *argv[])
{
	int arg_idx;
	Fl::args(argc, argv, arg_idx, parse_args);

	check_homedir();

	progstatus.loadLastState();

	Fl::lock();

	mainwin = make_main_dialog();
	mainwin->callback((Fl_Callback *)cb_exit);
	mainwin->resize(
		progstatus.mainX, progstatus.mainY,
		progstatus.mainW, progstatus.mainH);

	commswin = make_comms_dialog();
	commswin->callback((Fl_Callback *)cb_comms_dialog);

	helpwin = make_info_dialog();
	helpwin->callback((Fl_Callback *)cb_helpwin);

	static std::string str_about;
	str_about.assign(PACKAGE_STRING).append("\n");
	str_about.append(PACKAGE_VENDOR);
	txt_help->addstr(str_about);

	init_port_combos();
	sel_comm_port->value(progstatus.serial_port.c_str());
	sel_comm_baud->value(progstatus.serial_baud.c_str());

	Init_Serial();
	Fl::add_timeout(0, start_digi_loop);

	mainwin->show();

	return Fl::run();

}
