// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of UBITX
//
// fldigi is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <string>

#include <FL/Fl_Preferences.H>

#include "config.h"

#include "flio.h"
#include "flio_status.h"

#define status_FILENAME "flio_status"

status::status() 
{
	mainX = 50;
	mainY = 50;
	mainW = 600;
	mainH = 400;

	serial_port = "NONE";
	serial_baud = "9600";
	handshake = "NONE";
	stopbits = "1";

	server_addr = "127.0.0.1";
	server_port = "7362";
	data_type = "Both";
}

void status::saveLastState()
{
	int mX = mainwin->x();
	int mY = mainwin->y();
	if (mX >= 0 && mX >= 0) {
		mainX = mX;
		mainY = mY;
	}
	mainW = mainwin->w();
	mainH = mainwin->h();

	Fl_Preferences spref(HomeDir.c_str(), PACKAGE_VENDOR, PACKAGE_TARNAME);

	spref.set("version", PACKAGE_VERSION);

	spref.set("main_x", mainX);
	spref.set("main_y", mainY);
	spref.set("main_w", mainW);
	spref.set("main_h", mainH);

	spref.set("serial_port", serial_port.c_str());
	spref.set("serial_baud", serial_baud.c_str());
	spref.set("serial_handshake", handshake.c_str());
	spref.set("serial_stopbits", stopbits.c_str());

	spref.set("server_addr", server_addr.c_str());
	spref.set("server_port", server_port.c_str());

}

void status::loadLastState()
{
	Fl_Preferences spref(HomeDir.c_str(), PACKAGE_VENDOR, PACKAGE_TARNAME);

	char strbuff[500];
	char version[64];
	version[sizeof(version)-1] = '\0';

	if (!spref.get("version", version, "", sizeof(version)-1))
		return;

	spref.get("main_x", mainX, mainX);
	if (mainX > Fl::w()) mainX = 50;

	spref.get("main_y", mainY, mainY);
	if (mainY > Fl::h()) mainY = 50;

	spref.get("main_w", mainW, mainW);
	spref.get("main_h", mainH, mainH);

	memset(strbuff, 0, sizeof(strbuff));
	spref.get("serial_port", strbuff, "NONE", 499);
	serial_port = strbuff;

	if (serial_port.find("tty") == 0)
		serial_port.insert(0, "/dev/");

	spref.get("serial_baud", strbuff, "NONE", 499);
	serial_baud = strbuff;

	spref.get("serial_handshake", strbuff, "NONE", 499);
	handshake = strbuff;

	spref.get("serial_stopbits", strbuff, "0", 499);
	stopbits = strbuff;

	spref.get("server_addr", strbuff, "NONE", 499);
	server_addr = strbuff;

	spref.get("server_port", strbuff, "NONE", 499);
	server_port = strbuff;

}
